/***************************************************************************
 *   Copyright (C) 2005 by Florian Roth   *
 *   florian@synatic.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
 
#include <kglobalsettings.h>
#include <qfont.h>
#include <qpainter.h>
#include <kdebug.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kiconloader.h>

#include "metaframe.h"

MetaFrame::MetaFrame(QWidget *parent, const char *name) : QFrame(parent, name)
{
  timer = new QTimer();
  connect(timer, SIGNAL(timeout()), this, SLOT(animateResizeImpl()));

  setPaletteBackgroundColor(KGlobalSettings::baseColor());
  setPaletteForegroundColor(KGlobalSettings::highlightColor());
  setFrameShape(QFrame::Box);
  setFrameShadow(QFrame::Plain);
  
  titleLabel = new TitleLabel(this);
  connect(titleLabel, SIGNAL(toggled(bool)), this, SLOT(collapse(bool)));
  connect(titleLabel, SIGNAL(toggled(bool)), this, SIGNAL(toggled(bool)));
  
  main = new FadeWidget(this);
  main->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
  
  layout = new QVBoxLayout(main, 4, 5);
  
  QVBoxLayout *mainLayout = new QVBoxLayout(this, 1, 1);
  mainLayout->addWidget(titleLabel);
  mainLayout->addWidget(main);
}

MetaFrame::~MetaFrame(){
}

void MetaFrame::addWidget(QWidget *w, bool adjust)
{
  layout->addWidget(w);
  
  if(adjust){
    animateResize();
  }
}
    
void MetaFrame::setTitle(const QString &title)
{  
  titleLabel->setText(title);
}

const QString MetaFrame::title()
{
  return titleLabel->text();
}

void MetaFrame::setExpanded(bool b)
{
  if(b){
    main->show();
  }

  main->fadeIn(b);
  animateResize();
}

void MetaFrame::collapse(bool b)
{
  setExpanded(!b);
}

bool MetaFrame::isExpanded()
{
  return !titleLabel->toggled();
}

QWidget* MetaFrame::mainWidget()
{
  return main;
}

void MetaFrame::animateResize()
{
  main->updateGeometry();
  wishedHeight = isExpanded() ? main->sizeHint().height() : 0;
  
  if(!timer->isActive()){
    main->setFixedHeight(main->height());
    
    timer->start(8);
  }
}

void MetaFrame::animateResizeImpl()
{
  int currentHeight = main->height();
  int scrollAmount = currentHeight < wishedHeight ? 3 : -3;
  
  
  int newHeight = scrollAmount > 0 ? kMin(currentHeight + scrollAmount, wishedHeight) : kMax(currentHeight + scrollAmount, wishedHeight); 
  
  main->setFixedHeight(newHeight);
    
  if((scrollAmount / kAbs(scrollAmount)) *newHeight >= wishedHeight){
    if(!isExpanded()){
      main->hide();
    }
    
    timer->stop();
  }
}

//Fade widget
FadeWidget::FadeWidget(QWidget *parent, const char *name) : QWidget(parent, name)
{
  timer = new QTimer();
}

FadeWidget::~FadeWidget()
{
  delete timer;
}

void FadeWidget::fadeIn(bool b)
{
  if(b){
    fadeIn();
  }
  else{
    fadeOut();
  } 
}

void FadeWidget::fadeIn()
{

}

void FadeWidget::fadeOut()
{

}

void FadeWidget::paintEvent(QPaintEvent *e)
{
  QWidget::paintEvent(e);

/*  
  //code based on kopete's contact list fading
#ifdef HAVE_XRENDER
  QPixmap back(width(), height());
  QPainter paint(&back);
  QPainter painter(this);
  
  QPainter *p = &paint;  
  paint.end();

  //QColor rgb = KGlobalSettings::baseColor();//backgroundColor();
  QColor rgb(255, 0, 0);
  
  int visibilityLevel = 5;
  
  float opac = 1.0;
  //if (timer.isActive()){
    int vis = QMAX( visibilityLevel, 0 );
    opac = float(vis) / 10; //fade steps
  //}
  
  opac *= 0.5; //opacity
  const int alpha = 257 - int(opac * 257);
  if ( alpha != 0 )
  {
    XRenderColor clr = { alpha * rgb.red(), alpha * rgb.green(), alpha * rgb.blue(), alpha * 0xff };
    XRenderFillRectangle(back.x11Display(), PictOpOver, back.x11RenderHandle(), &clr, 0, 0, width(), height());
  }

  p->drawPixmap(0, 0, back);
  
  painter.fillRect(0, 0, width(), height(), QColor(255, 0, 0));
  painter.drawPixmap(0, 0, back);
  painter.end();
#endif
*/
}

TitleLabel::TitleLabel(QWidget *parent, const char *name) : QLabel(parent, name)
{
  setPaletteForegroundColor(KGlobalSettings::highlightedTextColor());
  
  QFont font = this->font();
  font.setBold(true);
  setFont(font);
  
  setMargin(2);
  
  isToggled = false;
}

TitleLabel::~TitleLabel()
{
}

void TitleLabel::setToggled(const bool b)
{
  if(isToggled != b){  
    isToggled = b;
    emit toggled(b);
    
    update();
  }
}

const bool TitleLabel::toggled()
{
  return isToggled;
}

void TitleLabel::paintEvent(QPaintEvent *e)
{
  KPixmap pix;
  pix.resize(width(), 10);
  pix = KPixmapEffect::gradient(pix, KGlobalSettings::highlightColor().light(), KGlobalSettings::highlightColor(), KPixmapEffect::VerticalGradient);

  QPainter p(this);
  p.drawPixmap(0, 0, pix);
  p.fillRect(0, 10, width(), height() - 10, KGlobalSettings::highlightColor());
  
  QPixmap led = isToggled ? SmallIcon("metabardown") : SmallIcon("metabarup");
  int x = width() - led.width() - 2;
  int y = (height() - led.height()) / 2;
  p.drawPixmap(x, y, led);
  
  p.end();

  QLabel::paintEvent(e);
}

void TitleLabel::mouseReleaseEvent(QMouseEvent *e)
{
  setToggled(!isToggled);
}
